﻿using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class Run : CodeNode {
    public override int MinimumArity { get { return 2; } }
    public override int MaximumArity { get { return 10; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    private Run(bool deserializing) : base(deserializing) { }
    private Run(Run original, Cloner cloner)
      : base(original, cloner) {
    }

    public Run()
      : base("Run", "The main method in every robot.") {
      this.Prefix = "\r\npublic void run() {\r\n\tsetAdjustGunForRobotTurn(true);\r\n\tturnRadarRightRadians(Double.POSITIVE_INFINITY);";
      this.Suffix = "\r\nexecute();\r\n}\r\n";
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Run(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, System.Collections.Generic.IEnumerable<ISymbolicExpressionTreeNode> children) {
      string result = "";
      foreach (ISymbolicExpressionTreeNode c in children)
        result += "\r\n" + ((CodeNode)c.Symbol).Interpret(c, c.Subtrees);
      return this.Prefix + "\r\n" + result + "\r\n" + this.Suffix;
    }
  }
}